@@ -52,7 +52,7 @@ SV *Py2Pl(PyObject * obj) {
 #endif
 	/* elw: this needs to be early */
 	/* None (like undef) */
-	if (obj == Py_None) {
+	if (!obj || obj == Py_None) {
 		Printf(("Py2Pl: Py_None\n"));
 		return &PL_sv_undef;
 	}
@@ -101,7 +101,9 @@ SV *Py2Pl(PyObject * obj) {
 		/*SvREADONLY_on(inst); *//* to uncomment this means I can't
 			re-bless it */
 		Py_INCREF(obj);
-		Printf(("Py2Pl: Instance\n"));
+		Printf(("Py2Pl: Instance. Obj: %p, inst_ptr: %p\n", obj, inst_ptr));
+
+		sv_2mortal(inst_ptr);
 		return inst_ptr;
 	}
 
@@ -117,6 +119,7 @@ SV *Py2Pl(PyObject * obj) {
 			PyObject *tmp = PySequence_GetItem(obj, i);	/* new reference */
 			SV *next = Py2Pl(tmp);
 			av_push(retval, next);
+			SvREFCNT_inc(next);
 			Py_DECREF(tmp);
 		}
 		return newRV_noinc((SV *) retval);
@@ -172,6 +175,7 @@ SV *Py2Pl(PyObject * obj) {
 			}
 
 			hv_store(retval, key_val, strlen(key_val), sv_val, 0);
+			SvREFCNT_inc(sv_val);
 			Py_DECREF(key);
 			Py_DECREF(val);
 		}
@@ -183,6 +187,10 @@ SV *Py2Pl(PyObject * obj) {
 	/* a string (or number) */
 	else {
 		PyObject *string = PyObject_Str(obj);	/* new reference */
+		if (!string) {
+			Printf(("Py2Pl: string is NULL!? -> Py_None\n"));
+			return &PL_sv_undef;
+		}
 		char *str = PyString_AsString(string);
 		SV *s2 = newSVpv(str, PyString_Size(string));
 		Printf(("Py2Pl: string / number\n"));
@@ -217,10 +225,11 @@ PyObject *Pl2Py(SV * obj) {
 			IV ptr = SvIV(obj_deref);
 			if (!ptr) {
 				croak
-					("Internal error: Pl2Py() caught NULL PyObject* at %s, line %s.\n",
+					("Internal error: Pl2Py() caught NULL PyObject* at %s, line %i.\n",
 					 __FILE__, __LINE__);
 			}
 			o = (PyObject *) ptr;
+			Py_INCREF(o);
 		}
 		else {
 			HV *stash = SvSTASH(obj_deref);
@@ -228,8 +237,8 @@ PyObject *Pl2Py(SV * obj) {
 			SV *full_pkg = newSVpvf("main::%s::", pkg);
 			PyObject *pkg_py;
 
-			Printf(("A Perl object (%s). Wrapping...\n",
-					SvPV(full_pkg, PL_na)));
+			Printf(("A Perl object (%s, refcnt: %i). Wrapping...\n",
+					SvPV(full_pkg, PL_na), SvREFCNT(obj)));
 
 			pkg_py = PyString_FromString(SvPV(full_pkg, PL_na));
 			o = newPerlObj_object(obj, pkg_py);
@@ -255,6 +264,7 @@ PyObject *Pl2Py(SV * obj) {
 			fprintf(stderr, "your Perl string \"%s\" could not \n",
 					SvPV_nolen(obj));
 			fprintf(stderr, "be converted to a Python string\n");
+			o = PyFloat_FromDouble((double) 0);
 		}
 		Py_DECREF(tmp);
 	}
@@ -277,7 +287,7 @@ PyObject *Pl2Py(SV * obj) {
 		Printf(("array (%i)\n", len));
 
 		for (i = 0; i < len; i++) {
-			SV *tmp = av_shift(av);
+			SV *tmp = *av_fetch(av, i, 0);
 			PyTuple_SetItem(o, i, Pl2Py(tmp));
 		}
 	}
